@@ -9,11 +9,6 @@ import (
 	"testing"
 	"time"
 
-	v1 "github.com/openshift/api/samples/v1"
-	"github.com/openshift/cluster-samples-operator/pkg/cache"
-	operator "github.com/openshift/cluster-samples-operator/pkg/operatorstatus"
-	"github.com/openshift/cluster-samples-operator/pkg/util"
-
 	corev1 "k8s.io/api/core/v1"
 	kerrors "k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
@@ -24,9 +19,13 @@ import (
 	configv1 "github.com/openshift/api/config/v1"
 	imagev1 "github.com/openshift/api/image/v1"
 	operatorsv1api "github.com/openshift/api/operator/v1"
+	v1 "github.com/openshift/api/samples/v1"
 	templatev1 "github.com/openshift/api/template/v1"
-
 	imagev1client "github.com/openshift/client-go/image/clientset/versioned/typed/image/v1"
+
+	"github.com/openshift/cluster-samples-operator/pkg/cache"
+	operator "github.com/openshift/cluster-samples-operator/pkg/operatorstatus"
+	"github.com/openshift/cluster-samples-operator/pkg/util"
 )
 
 const (
@@ -71,7 +70,7 @@ func TestWithDist(t *testing.T) {
 	// image in progress (4th entry, array index 3) should still be false when there is no content ... a la z or ppc
 	statuses = []corev1.ConditionStatus{corev1.ConditionFalse, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}
 	validate(true, err, "", cfg, conditions, statuses, t)
-	mimic(&h, x86OCPContentRootDir)
+	mimic(&h, x86ContentRootDir)
 	err = h.Handle(event)
 	// with content present, image im progress should now be true
 	statuses = []corev1.ConditionStatus{corev1.ConditionFalse, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}
@@ -91,7 +90,7 @@ func TestWithArchDist(t *testing.T) {
 		t.Errorf("arch set to %s instead of %s", cfg.Spec.Architectures[0], runtime.GOARCH)
 	}
 
-	mimic(&h, x86OCPContentRootDir)
+	mimic(&h, x86ContentRootDir)
 	err := h.Handle(event)
 	statuses := []corev1.ConditionStatus{corev1.ConditionFalse, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}
 	validate(true, err, "", cfg,
@@ -128,7 +127,7 @@ func TestManagementState(t *testing.T) {
 	h, cfg, event := setup()
 	iskeys := getISKeys()
 	tkeys := getTKeys()
-	mimic(&h, x86OCPContentRootDir)
+	mimic(&h, x86ContentRootDir)
 	cfg.Spec.ManagementState = operatorsv1api.Unmanaged
 
 	err := h.Handle(event)
@@ -217,7 +216,7 @@ func TestSkipped(t *testing.T) {
 	cfg.Status.SkippedImagestreams = iskeys
 	cfg.Status.SkippedTemplates = tkeys
 
-	mimic(&h, x86OCPContentRootDir)
+	mimic(&h, x86ContentRootDir)
 
 	err := h.Handle(event)
 	validate(true, err, "", cfg, conditions, []corev1.ConditionStatus{corev1.ConditionFalse, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}, t)
@@ -299,7 +298,7 @@ func TestProcessed(t *testing.T) {
 	iskeys := getISKeys()
 	tkeys := getTKeys()
 
-	mimic(&h, x86OCPContentRootDir)
+	mimic(&h, x86ContentRootDir)
 
 	err := h.Handle(event)
 	validate(true, err, "", cfg,
@@ -332,6 +331,10 @@ func TestProcessed(t *testing.T) {
 	if is == nil || !strings.HasPrefix(is.Spec.DockerImageRepository, cfg.Spec.SamplesRegistry) {
 		t.Fatalf("stream repo not updated %#v, %#v", is, h)
 	}
+	is, _ = fakeisclient.Get("baz")
+	if is == nil || !strings.HasPrefix(is.Spec.DockerImageRepository, cfg.Spec.SamplesRegistry) {
+		t.Fatalf("stream repo not updated %#v, %#v", is, h)
+	}
 
 	faketclient := h.templateclientwrapper.(*fakeTemplateClientWrapper)
 	for _, key := range tkeys {
@@ -363,9 +366,66 @@ func TestProcessed(t *testing.T) {
 	if is == nil || !strings.HasPrefix(is.Spec.DockerImageRepository, cfg.Spec.SamplesRegistry) {
 		t.Fatalf("stream repo not updated %#v, %#v", is, h)
 	}
+	is, _ = fakeisclient.Get("baz")
+	if is == nil || !strings.HasPrefix(is.Spec.DockerImageRepository, cfg.Spec.SamplesRegistry) {
+		t.Fatalf("stream repo not updated %#v, %#v", is, h)
+	}
 
-	// make sure registries are updated when switch back to default
+	// make sure registries are updated when sampleRegistry is the form of host/path
+	cfg.Spec.SamplesRegistry = "foo.io/bar"
 	cfg.ResourceVersion = "3"
+	// fake out that the samples completed updating
+	progressing = util.Condition(cfg, v1.ImageChangesInProgress)
+	progressing.Status = corev1.ConditionFalse
+	util.ConditionUpdate(cfg, progressing)
+	// reset operator image to clear out previous registry image override
+	mimic(&h, x86ContentRootDir)
+
+	err = h.Handle(event)
+	validate(true, err, "", cfg,
+		conditions,
+		[]corev1.ConditionStatus{corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}, t)
+	is, _ = fakeisclient.Get("foo")
+	if is == nil || !strings.HasPrefix(is.Spec.DockerImageRepository, cfg.Spec.SamplesRegistry) {
+		t.Fatalf("stream repo not updated %#v, %#v", is, h)
+	}
+	is, _ = fakeisclient.Get("bar")
+	if is == nil || !strings.HasPrefix(is.Spec.DockerImageRepository, cfg.Spec.SamplesRegistry) {
+		t.Fatalf("stream repo not updated %#v, %#v", is, h)
+	}
+	is, _ = fakeisclient.Get("baz")
+	if is == nil || !strings.HasPrefix(is.Spec.DockerImageRepository, cfg.Spec.SamplesRegistry) {
+		t.Fatalf("stream repo not updated %#v, %#v", is, h)
+	}
+
+	// make sure registries are updated when sampleRegistry is the form of host:port/path
+	cfg.Spec.SamplesRegistry = "foo.io:1111/bar"
+	cfg.ResourceVersion = "4"
+	// fake out that the samples completed updating
+	progressing = util.Condition(cfg, v1.ImageChangesInProgress)
+	progressing.Status = corev1.ConditionFalse
+	util.ConditionUpdate(cfg, progressing)
+	// reset operator image to clear out previous registry image override
+	mimic(&h, x86ContentRootDir)
+
+	err = h.Handle(event)
+	validate(true, err, "", cfg,
+		conditions,
+		[]corev1.ConditionStatus{corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}, t)
+	is, _ = fakeisclient.Get("foo")
+	if is == nil || !strings.HasPrefix(is.Spec.DockerImageRepository, cfg.Spec.SamplesRegistry) {
+		t.Fatalf("stream repo not updated %#v, %#v", is, h)
+	}
+	is, _ = fakeisclient.Get("bar")
+	if is == nil || !strings.HasPrefix(is.Spec.DockerImageRepository, cfg.Spec.SamplesRegistry) {
+		t.Fatalf("stream repo not updated %#v, %#v", is, h)
+	}
+	is, _ = fakeisclient.Get("baz")
+	if is == nil || !strings.HasPrefix(is.Spec.DockerImageRepository, cfg.Spec.SamplesRegistry) {
+		t.Fatalf("stream repo not updated %#v, %#v", is, h)
+	}
+	// make sure registries are updated when switch back to default
+	cfg.ResourceVersion = "5"
 	cfg.Spec.SamplesRegistry = ""
 	// also make sure processing occurs for disruptive config change even if progressing==true
 	progressing = util.Condition(cfg, v1.ImageChangesInProgress)
@@ -373,7 +433,7 @@ func TestProcessed(t *testing.T) {
 	progressing.Status = corev1.ConditionFalse
 	util.ConditionUpdate(cfg, progressing)
 	// reset operator image to clear out previous registry image override
-	mimic(&h, x86OCPContentRootDir)
+	mimic(&h, x86ContentRootDir)
 
 	err = h.Handle(event)
 	is, _ = fakeisclient.Get("foo")
@@ -384,12 +444,15 @@ func TestProcessed(t *testing.T) {
 	if is == nil || strings.HasPrefix(is.Spec.DockerImageRepository, "bar.io") {
 		t.Fatalf("bar stream repo still has bar.io")
 	}
-
+	is, _ = fakeisclient.Get("baz")
+	if is == nil || !strings.HasPrefix(is.Spec.DockerImageRepository, cfg.Spec.SamplesRegistry) {
+		t.Fatalf("stream repo not updated %#v, %#v", is, h)
+	}
 }
 
 func TestImageStreamEvent(t *testing.T) {
 	h, cfg, event := setup()
-	mimic(&h, x86OCPContentRootDir)
+	mimic(&h, x86ContentRootDir)
 	err := h.Handle(event)
 	statuses := []corev1.ConditionStatus{corev1.ConditionFalse, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}
 	validate(true, err, "", cfg, conditions, statuses, t)
@@ -482,12 +545,41 @@ func TestImageStreamEvent(t *testing.T) {
 		},
 	}
 	h.processImageStreamWatchEvent(is, false)
+	is = &imagev1.ImageStream{
+		ObjectMeta: metav1.ObjectMeta{
+			Name: "baz",
+			Annotations: map[string]string{
+				v1.SamplesVersionAnnotation: h.version,
+			},
+		},
+		Spec: imagev1.ImageStreamSpec{
+			Tags: []imagev1.TagReference{
+				{
+					Name:       "baz",
+					Generation: &tagVersion,
+				},
+			},
+		},
+		Status: imagev1.ImageStreamStatus{
+			Tags: []imagev1.NamedTagEventList{
+				{
+					Tag: "baz",
+					Items: []imagev1.TagEvent{
+						{
+							Generation: tagVersion,
+						},
+					},
+				},
+			},
+		},
+	}
+	h.processImageStreamWatchEvent(is, false)
 	validate(true, err, "", cfg, conditions, statuses, t)
 }
 
 func TestImageStreamErrorRetry(t *testing.T) {
 	h, cfg, event := setup()
-	mimic(&h, x86OCPContentRootDir)
+	mimic(&h, x86ContentRootDir)
 	err := h.Handle(event)
 	statuses := []corev1.ConditionStatus{corev1.ConditionFalse, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}
 	validate(true, err, "", cfg, conditions, statuses, t)
@@ -602,7 +694,7 @@ func TestImageStreamErrorRetry(t *testing.T) {
 
 func TestTemplateEvent(t *testing.T) {
 	h, cfg, event := setup()
-	mimic(&h, x86OCPContentRootDir)
+	mimic(&h, x86ContentRootDir)
 	err := h.Handle(event)
 	statuses := []corev1.ConditionStatus{corev1.ConditionFalse, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}
 	validate(true, err, "", cfg, conditions, statuses, t)
@@ -638,7 +730,7 @@ func setup() (Handler, *v1.Config, util.Event) {
 
 func TestTemplateRemovedFromPayload(t *testing.T) {
 	h, _, _ := setup()
-	mimic(&h, x86OCPContentRootDir)
+	mimic(&h, x86ContentRootDir)
 
 	_, filePath, doUpsert, updateCfgOnly, err := h.prepSamplesWatchEvent("template", "no-longer-exists", map[string]string{}, false)
 	switch {
@@ -655,7 +747,7 @@ func TestTemplateRemovedFromPayload(t *testing.T) {
 
 func TestImageStreamRemovedFromPayloadWithProgressingErrors(t *testing.T) {
 	h, cfg, _ := setup()
-	mimic(&h, x86OCPContentRootDir)
+	mimic(&h, x86ContentRootDir)
 	progressing := util.Condition(cfg, v1.ImageChangesInProgress)
 	progressing.Reason = "foo "
 	errors := util.Condition(cfg, v1.ImportImageErrorsExist)
@@ -713,7 +805,7 @@ func TestImageStreamRemovedFromPayloadWithProgressingErrors(t *testing.T) {
 func TestImageStreamCreateErrorDegradedReason(t *testing.T) {
 	err := kerrors.NewServiceUnavailable("BadService")
 	h, cfg, event := setup()
-	mimic(&h, x86OCPContentRootDir)
+	mimic(&h, x86ContentRootDir)
 	fakeisclient := h.imageclientwrapper.(*fakeImageStreamClientWrapper)
 	fakeisclient.geterrors = map[string]error{"foo": err}
 	h.Handle(event)
@@ -732,7 +824,7 @@ func TestImageGetError(t *testing.T) {
 	for _, iserr := range errors {
 		h, cfg, event := setup()
 
-		mimic(&h, x86OCPContentRootDir)
+		mimic(&h, x86ContentRootDir)
 
 		fakeisclient := h.imageclientwrapper.(*fakeImageStreamClientWrapper)
 		fakeisclient.geterrors = map[string]error{"foo": iserr}
@@ -754,7 +846,7 @@ func TestImageUpdateError(t *testing.T) {
 
 	h, cfg, event := setup()
 
-	mimic(&h, x86OCPContentRootDir)
+	mimic(&h, x86ContentRootDir)
 
 	fakeisclient := h.imageclientwrapper.(*fakeImageStreamClientWrapper)
 	fakeisclient.upserterrors = map[string]error{"foo": kerrors.NewServiceUnavailable("upsertstreamerror")}
@@ -916,10 +1008,9 @@ func TestImageStreamImportError(t *testing.T) {
 			},
 		},
 	}
-	turnBackThreeHours := true
 	for _, is := range streams {
 		h, cfg, _ := setup()
-		mimic(&h, x86OCPContentRootDir)
+		mimic(&h, x86ContentRootDir)
 		dir := h.GetBaseDir(v1.X86Architecture, cfg)
 		files, _ := h.Filefinder.List(dir)
 		h.processFiles(dir, files, cfg)
@@ -942,20 +1033,6 @@ func TestImageStreamImportError(t *testing.T) {
 		if len(importErr.Reason) == 0 || !util.NameInReason(cfg, importErr.Reason, is.Name) {
 			t.Fatalf("processImageStreamWatchEvent did not set import error reason field %#v for stream %#v", cfg, is)
 		}
-		if turnBackThreeHours {
-			importErr.LastTransitionTime.Time = metav1.Now().Add(-3 * time.Hour)
-		} else {
-			importErr.LastTransitionTime = metav1.Now()
-		}
-		util.ConditionUpdate(cfg, importErr)
-		status, reason, detail := util.ClusterOperatorStatusDegradedCondition(cfg)
-		if (status != configv1.ConditionTrue && turnBackThreeHours) || (status == configv1.ConditionTrue && !turnBackThreeHours) {
-			t.Fatalf("image import error from %#v not reflected in cluster status %#v", is, cfg)
-		}
-		if status == configv1.ConditionFalse && turnBackThreeHours && (len(reason) == 0 || len(detail) == 0) {
-			t.Fatalf("image import error from is %#v with cfg %#v had reason %s and detail %s", is, cfg, reason, detail)
-		}
-		turnBackThreeHours = !turnBackThreeHours
 	}
 }
 
@@ -1003,7 +1080,7 @@ func TestImageStreamImportErrorRecovery(t *testing.T) {
 		},
 	}
 	h, cfg, _ := setup()
-	mimic(&h, x86OCPContentRootDir)
+	mimic(&h, x86ContentRootDir)
 	dir := h.GetBaseDir(v1.X86Architecture, cfg)
 	files, _ := h.Filefinder.List(dir)
 	h.processFiles(dir, files, cfg)
@@ -1076,7 +1153,7 @@ func TestTemplateGetError(t *testing.T) {
 	for _, terr := range errors {
 		h, cfg, event := setup()
 
-		mimic(&h, x86OCPContentRootDir)
+		mimic(&h, x86ContentRootDir)
 
 		faketclient := h.templateclientwrapper.(*fakeTemplateClientWrapper)
 		faketclient.geterrors = map[string]error{"bo": terr}
@@ -1097,7 +1174,7 @@ func TestTemplateGetError(t *testing.T) {
 func TestTemplateUpsertError(t *testing.T) {
 	h, cfg, event := setup()
 
-	mimic(&h, x86OCPContentRootDir)
+	mimic(&h, x86ContentRootDir)
 
 	faketclient := h.templateclientwrapper.(*fakeTemplateClientWrapper)
 	faketclient.upserterrors = map[string]error{"bo": kerrors.NewServiceUnavailable("upsertstreamerror")}
@@ -1119,7 +1196,7 @@ func TestDeletedCR(t *testing.T) {
 
 func TestSameCR(t *testing.T) {
 	h, cfg, event := setup()
-	mimic(&h, x86OCPContentRootDir)
+	mimic(&h, x86ContentRootDir)
 	cfg.ResourceVersion = "a"
 
 	// first pass on the resource creates the samples, exists (first entry, index 0) is still false
@@ -1141,7 +1218,7 @@ func TestSameCR(t *testing.T) {
 func TestBadTopDirList(t *testing.T) {
 	h, cfg, event := setup()
 	fakefinder := h.Filefinder.(*fakeResourceFileLister)
-	fakefinder.errors = map[string]error{x86OCPContentRootDir: fmt.Errorf("badtopdir")}
+	fakefinder.errors = map[string]error{x86ContentRootDir: fmt.Errorf("badtopdir")}
 	err := h.Handle(event)
 	statuses := []corev1.ConditionStatus{corev1.ConditionUnknown, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}
 	validate(false, err, "badtopdir", cfg, conditions, statuses, t)
@@ -1149,9 +1226,9 @@ func TestBadTopDirList(t *testing.T) {
 
 func TestBadSubDirList(t *testing.T) {
 	h, cfg, event := setup()
-	mimic(&h, x86OCPContentRootDir)
+	mimic(&h, x86ContentRootDir)
 	fakefinder := h.Filefinder.(*fakeResourceFileLister)
-	fakefinder.errors = map[string]error{x86OCPContentRootDir + "/imagestreams": fmt.Errorf("badsubdir")}
+	fakefinder.errors = map[string]error{x86ContentRootDir + "/imagestreams": fmt.Errorf("badsubdir")}
 	err := h.Handle(event)
 	statuses := []corev1.ConditionStatus{corev1.ConditionUnknown, corev1.ConditionTrue, corev1.ConditionTrue, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse, corev1.ConditionFalse}
 	validate(false, err, "badsubdir", cfg, conditions, statuses, t)
@@ -1163,7 +1240,7 @@ func TestBadSubDirList(t *testing.T) {
 
 func TestBadTopLevelStatus(t *testing.T) {
 	h, cfg, event := setup()
-	mimic(&h, x86OCPContentRootDir)
+	mimic(&h, x86ContentRootDir)
 	fakestatus := h.crdwrapper.(*fakeCRDWrapper)
 	fakestatus.updateerr = fmt.Errorf("badsdkupdate")
 	err := h.Handle(event)
@@ -1183,7 +1260,7 @@ func invalidConfig(t *testing.T, msg string, cfgValid *v1.ConfigCondition) {
 }
 
 func getISKeys() []string {
-	return []string{"foo", "bar"}
+	return []string{"foo", "bar", "baz"}
 }
 
 func getTKeys() []string {
@@ -1215,6 +1292,10 @@ func mimic(h *Handler, topdir string) {
 				name: "bar",
 				dir:  false,
 			},
+			{
+				name: "baz",
+				dir:  false,
+			},
 		},
 		topdir + "/" + "templates": {
 			{
@@ -1262,11 +1343,30 @@ func mimic(h *Handler, topdir string) {
 			},
 		},
 	}
+	baz := &imagev1.ImageStream{
+		ObjectMeta: metav1.ObjectMeta{
+			Name:   "baz",
+			Labels: map[string]string{},
+		},
+		Spec: imagev1.ImageStreamSpec{
+			DockerImageRepository: registry2,
+			Tags: []imagev1.TagReference{
+				{
+					From: &corev1.ObjectReference{
+						Name: registry2 + "/repo/image:1.0",
+						Kind: "DockerImage",
+					},
+				},
+			},
+		},
+	}
 	fakeisgetter.streams = map[string]*imagev1.ImageStream{
 		topdir + "/imagestreams/foo": foo,
 		topdir + "/imagestreams/bar": bar,
+		topdir + "/imagestreams/baz": baz,
 		"foo":                        foo,
 		"bar":                        bar,
+		"baz":                        baz,
 	}
 	faketempgetter := h.Filetemplategetter.(*fakeTemplateFromFileGetter)
 	bo := &templatev1.Template{
